from .conversion.latin import VOWELS, CONSONANTS


def separate(text: str) -> list[str]:
    """Separate a romanized Ainu word into syllables.

    Args:
        text (str): A string that holds the romanized Ainu word.

    Returns:
        str: A list of syllables.
    """
    syllable_map = {}
    syllable_count = 1

    for i, char in enumerate(text):
        if char in VOWELS:
            if i > 0 and text[i - 1] in CONSONANTS:
                syllable_map[i - 1] = syllable_count
            syllable_map[i] = syllable_count
            syllable_count += 1

    for i in range(len(text)):
        if i not in syllable_map:
            syllable_map[i] = syllable_map.get(i - 1, 0)

    syllables = []
    current_group_id = 1
    head = 0

    for i in range(len(text)):
        if syllable_map.get(i, 0) != current_group_id:
            current_group_id = syllable_map.get(i, 0)
            syllables.append(text[head:i])
            head = i

    syllables.append(text[head:])

    return [s.replace("'", "") for s in syllables]



from enum import Enum


def is_cyrillic(char: str) -> bool:
    return "\u0400" <= char <= "\u04FF"


def is_katakana(char: str) -> bool:
    return "\u30A1" <= char <= "\u31FF"


class Script(Enum):
    """An enumeration of the possible script types for the Ainu language."""

    """The Katakana script."""
    Kana = "Kana"

    """The Cyrillic script."""
    Cyrl = "Cyrl"

    """The Latin script."""
    Latn = "Latn"

    """A mixed script."""
    Mixed = "Mixed"

    """An unknown script."""
    Unknown = "Unknown"


def detect(text: str) -> Script:
    """Detects the script type of a given Ainu language string.

    This function categorizes the script into one of several types based on the characters present in the string.

    It supports Latin, Cyrillic, Katakana scripts, and can also identify mixed or unknown scripts.

    Args:
        text (str): The text string to be analyzed for script type.

    Returns:
        Script: The detected script type:
            Kana for Katakana
            Cyrl for Cyrillic
            Latn for Latin
            Mixed if multiple scripts are detected
            Unknown if no script is detected
    """
    has_latin = any(c.isalpha() and c.isascii() for c in text)
    has_cyrillic = any(c.isalpha() and is_cyrillic(c) for c in text)
    has_kana = any(c.isalpha() and is_katakana(c) for c in text)

    if [has_latin, has_cyrillic, has_kana].count(True) > 1:
        return Script.Mixed
    elif has_kana:
        return Script.Kana
    elif has_cyrillic:
        return Script.Cyrl
    elif has_latin:
        return Script.Latn
    else:
        return Script.Unknown



from more_itertools import peekable

from ..conversion.latin import CONSONANTS
from ..syllable import separate

KANA_2_LATN = {
    "ア": "a",
    "イ": "i",
    "ウ": "u",
    "エ": "e",
    "オ": "o",
    "カ": "ka",
    "キ": "ki",
    "ク": "ku",
    "ケ": "ke",
    "コ": "ko",
    "サ": "sa",
    "シ": "si",
    "ス": "su",
    "セ": "se",
    "ソ": "so",
    "タ": "ta",
    "テ": "te",
    "ト": "to",
    "ナ": "na",
    "ニ": "ni",
    "ヌ": "nu",
    "ネ": "ne",
    "ノ": "no",
    "ハ": "ha",
    "ヒ": "hi",
    "フ": "hu",
    "ヘ": "he",
    "ホ": "ho",
    "パ": "pa",
    "ピ": "pi",
    "プ": "pu",
    "ペ": "pe",
    "ポ": "po",
    "マ": "ma",
    "ミ": "mi",
    "ム": "mu",
    "メ": "me",
    "モ": "mo",
    "ヤ": "ya",
    "ユ": "yu",
    "ヨ": "yo",
    "ラ": "ra",
    "リ": "ri",
    "ル": "ru",
    "レ": "re",
    "ロ": "ro",
    "ワ": "wa",
    "ヲ": "wo",
    "ン": "n",
    "ッ": "t",
    "ㇳ": "t",
    "ゥ": "w",
    "ㇰ": "k",
    "ㇵ": "x",
    "ㇶ": "x",
    "ㇷ": "x",
    "ㇸ": "x",
    "ㇹ": "x",
    "ァ": "a",
    "ェ": "e",
    "ォ": "o",
    "ㇻ": "r",
    "ㇼ": "r",
    "ㇽ": "r",
    "ㇾ": "r",
    "ㇿ": "r",
    "ィ": "y",
    "ㇷ": "h",
    "ㇱ": "s",
    "ㇺ": "m",
    "ㇴ": "n",
}

KANA_2_LATN_DIAGRAPH = {
    "イェ": "ye",
    "ウェ": "we",
    "ウィ": "wi",
    "ウォ": "wo",
    "トゥ": "tu",
    "ㇷ゚": "p",
    "ト゚": "tu",
    "チャ": "ca",
    "チュ": "cu",
    "チェ": "ce",
    "チョ": "co",
}


def kana2latn(kana: str) -> str:
    """Converts Katakana script to Latin script."""
    result = []
    chars = peekable(kana)

    for current_char in chars:
        latn = None
        next_char = chars.peek() if chars else None

        print(current_char, next_char)

        if (
            current_char
            and next_char
            and current_char + next_char in KANA_2_LATN_DIAGRAPH
        ):
            print(f"diagraph: {current_char + next_char = }")
            next(chars)
            latn = KANA_2_LATN_DIAGRAPH.get(current_char + next_char)
        else:
            latn = KANA_2_LATN.get(current_char)

        print(f"{latn = }")
        result.append(latn if latn is not None else current_char)

    return "".join(result)


def latn2kana(text: str) -> str:
    """Converts Latin script to Katakana script."""
    syllables = separate(text)

    result = ""

    for i, syllable in enumerate(syllables):
        next_syllable = syllables[i + 1] if i + 1 < len(syllables) else None
        if len(syllable) == 0:
            continue

        last_char = syllable[-1]

        if last_char in CONSONANTS:
            (remains, coda) = syllable[:-1], last_char
        else:
            (remains, coda) = syllable, ""

        converted_remains = {
            "a": "ア",
            "i": "イ",
            "u": "ウ",
            "e": "エ",
            "o": "オ",
            "'a": "ア",
            "'i": "イ",
            "'u": "ウ",
            "'e": "エ",
            "'o": "オ",
            "’a": "ア",
            "’i": "イ",
            "’u": "ウ",
            "’e": "エ",
            "’o": "オ",
            "ka": "カ",
            "ki": "キ",
            "ku": "ク",
            "ke": "ケ",
            "ko": "コ",
            "sa": "サ",
            "si": "シ",
            "su": "ス",
            "se": "セ",
            "so": "ソ",
            "ta": "タ",
            "tu": "ト゚",
            "te": "テ",
            "to": "ト",
            "ca": "チャ",
            "ci": "チ",
            "cu": "チュ",
            "ce": "チェ",
            "co": "チョ",
            "na": "ナ",
            "ni": "ニ",
            "nu": "ヌ",
            "ne": "ネ",
            "no": "ノ",
            "ha": "ハ",
            "hi": "ヒ",
            "hu": "フ",
            "he": "ヘ",
            "ho": "ホ",
            "pa": "パ",
            "pi": "ピ",
            "pu": "プ",
            "pe": "ペ",
            "po": "ポ",
            "ma": "マ",
            "mi": "ミ",
            "mu": "ム",
            "me": "メ",
            "mo": "モ",
            "ya": "ヤ",
            "yi": "イ",
            "yu": "ユ",
            "ye": "イェ",
            "yo": "ヨ",
            "ra": "ラ",
            "ri": "リ",
            "ru": "ル",
            "re": "レ",
            "ro": "ロ",
            "wa": "ワ",
            "wi": "ヰ",
            "we": "ヱ",
            "wo": "ヲ",
            "nn": "ン",
            "tt": "ッ",
        }.get(remains)

        result += converted_remains or ""

        print(f"{coda = } {next_syllable = }")

        next_char = next_syllable[0] if next_syllable else None

        converted_coda = {
            "w": "ゥ",
            "y": "ィ",
            "m": "ㇺ",
            "n": "ㇴ",
            "s": "ㇱ",
            "p": "ㇷ゚",
            "t": "ッ",
            "T": "ㇳ",
            "k": "ㇰ",
            "r": {
                "a": "ㇻ",
                "i": "ㇼ",
                "u": "ㇽ",
                "e": "ㇾ",
                "o": "ㇿ",
            }.get(next_char or "", "ㇽ"),
            "h": {
                "a": "ㇵ",
                "i": "ㇶ",
                "u": "ㇷ",
                "e": "ㇸ",
                "o": "ㇹ",
            }.get(next_char or "", "ㇷ"),
        }.get(coda, coda)

        result += converted_coda

    return (
        result.replace("ィ", "イ")
        .replace("ゥ", "ウ")
        .replace("ㇴ", "ン")
        .replace("ヱ", "ウェ")
        .replace("ヰ", "ウィ")
    )



from more_itertools import peekable

LATN_2_CYRL = {
    "a": "а",
    "i": "и",
    "u": "у",
    "e": "э",
    "o": "о",
    "k": "к",
    "s": "с",
    "t": "т",
    "c": "ц",
    "h": "х",
    "m": "м",
    "n": "н",
    "p": "п",
    "r": "р",
    "w": "в",
    "y": "й",
    "'": "ъ",
    "’": "",
}

LATN_2_CYRL_Y = {
    "u": "ю",
    "a": "я",
    "o": "ё",
    "e": "е",
}

LATN_2_CYRL_Y_WITH_Y = {f"y{k}": v for k, v in LATN_2_CYRL_Y.items()}

CYRL_2_LATN_Y = {
    "у": "y’u",
    "а": "y’a",
    "о": "y’o",
    "э": "y’e",
}


def latn2cyrl(text: str) -> str:
    """Converts Latin script to Cyrillic script."""
    result = []
    chars = peekable(text)

    for current_char in chars:
        current_lower = current_char.lower()
        cyrl = None
        next_char = chars.peek() if chars else None
        next_lower = next_char.lower() if next_char else None

        if current_lower == "y" and next_lower in LATN_2_CYRL_Y:
            next(chars)
            cyrl = LATN_2_CYRL_Y.get(next_lower)
            if cyrl:
                if current_char.isupper():
                    cyrl = cyrl.upper()
        else:
            cyrl = LATN_2_CYRL.get(current_lower)

        result.append(cyrl if cyrl is not None else current_char)

    return "".join(result)


def cyrl2latn(text: str) -> str:
    """Converts Cyrillic script to Latin script."""
    result = []
    chars = peekable(text)

    for current_char in chars:
        current_lower = current_char.lower()
        cyrl = None
        next_char = chars.peek() if chars else None
        next_lower = next_char.lower() if next_char else None

        print(f"f{current_lower = } {next_lower = }")

        if current_lower == "й" and next_lower in CYRL_2_LATN_Y:
            print("й with vowel")
            next(chars)
            cyrl = CYRL_2_LATN_Y.get(next_lower)
        else:
            print(LATN_2_CYRL | LATN_2_CYRL_Y)

            cyrl = next(
                (
                    latn
                    for latn, cyrl in (LATN_2_CYRL | LATN_2_CYRL_Y_WITH_Y).items()
                    if cyrl == current_lower
                ),
                None,
            )

        result.append(cyrl if cyrl is not None else current_char)

    return "".join(result)



from .conversion.cyrillic import latn2cyrl, cyrl2latn
from .conversion.katakana import latn2kana, kana2latn
from .syllable import separate
from .detection import detect, Script

__all__ = [
    "latn2kana",
    "kana2latn",
    "latn2cyrl",
    "cyrl2latn",
    "separate",
    "detect",
    "Script",
]
